/*
 * @file   client_socket.hh
 * @author David Weber <david.weber@octavosystems.com>
 * @date   02/14/2019
 *
 * @brief client_socket module implementation
 *
 * Copyright © 2019 Octavo Systems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __CLIENT_SOCKET_HH__
#define __CLIENT_SOCKET_HH__

#include <arpa/inet.h>
#include "socket.hh"
#include "rtstatus.h"

class client_socket : public socket
{
  private:
    sockaddr_in_t addr;
    char* ip_string; 

  public:
    client_socket(char* ip_string,
                  unsigned short port, 
                  unsigned short addr_family = AF_INET, 
                  int        protocol_family = PF_INET,  
                  int                   type = SOCK_STREAM, 
                  int               protocol = IPPROTO_TCP); 
    ~client_socket();
    rt_status_t connect();
    rt_status_t connect_non_block();
    ssize_t send(void* send_data, ssize_t send_len);
    ssize_t recv(void* recv_data, ssize_t recv_len);
    rt_status_t send_wait(void* send_data, ssize_t send_len);
    rt_status_t recv_wait(void* recv_data, ssize_t recv_len);
};

#endif // __CLIENT_SOCKET_HH__
      
