/*
 * @file   msg_client.cc
 * @author David Weber <david.weber@octavosystems.com>
 * @date   02/10/2019
 *
 * @brief msg_client module implementation
 *
 * Copyright © 2019 Octavo Systems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "socket.hh"
#include "client_socket.hh"
#include "msg_types.h"

int main(int argc, char* argv[])
{
  if (argc != 3)
  {
    printf("usage: %s <ip> <port>\n", PROG_NAME);
    return -1;
  }

  char* ip_string = argv[1];
  char* port_string = argv[2];

  int port = atoi(port_string);
  rt_status_t status;

  client_socket* cs = new client_socket(ip_string, port);
  if (cs == NULL)
  {
    printf("unable to allocate memory for client_socket\n");
    return -1;
  }
  else if (cs->open() != SUCCESS)
  {
    printf("unable to open socket\n");
    return -1;
  }
  else if (cs->connect() != SUCCESS)
  {
    printf("unable to connect to server %s on port %d\n", ip_string, port);
    return -1;
  }
  char data[2048];
  // do
  // {
  printf("#> \n");
  fgets(data, 2048, stdin);
  printf("data len = %lu\n", strlen(data));
  data[strlen(data) -1] = 0;  // remove new line
  printf("data len = %lu\n", strlen(data));
#if 0
  msg_header_t hdr =
  {
    .length = sizeof(msg_header_t) + strlen(data),
    .msg_id = 10,
    .src_id = 200,
    .seq_num = 1,
  };
#else
  msg_header_t hdr;
  hdr.length = sizeof(msg_header_t) + strlen(data),
  hdr.msg_id = 10,
  hdr.src_id = 200,
  hdr.seq_num = 1,
#endif
  printf("length = %lu\n", hdr.length);
  printf("msg_id = %u\n", hdr.msg_id);
  printf("src_id = %u\n", hdr.src_id);
  printf("seq_num = %u\n", hdr.seq_num);
  printf("data = %s\n", data);
  cs->send(&hdr, sizeof(msg_header_t));
  cs->send(data, strlen(data));
  // printf("sleeping...\n");
  // sleep(1);
  // }
  // while (strcmp("exit", data) != 0);
#if 0
  msg_header_t msg_header1 = { MSG_VER, 5, 0x1234 };
  msg_header1.crc = crc16(sizeof(msg_header_t) - 2, (uint8_t*)&msg_header1);
  msg_header_t msg_header2 = { MSG_CMD_STR, sizeof(cmd_string), 0x5544 };
  msg_header_t msg_quit =    { MSG_QUIT, 0, 0x8765 };
  msg_quit.crc = crc16(sizeof(msg_header_t) - sizeof(crc16_t), (uint8_t*)&msg_quit);

  cs->send(&msg_header1, sizeof(msg_header_t));
  cs->send(&msg_header2, sizeof(msg_header_t));
  cs->send(cmd_string, sizeof(cmd_string));
#endif
//  cs->send(&msg_quit, sizeof(msg_header_t));
  delete(cs);
}

