/*
 * @file   server_socket.hh
 * @author David Weber <david.weber@octavosystems.com>
 * @date   02/14/2019
 *
 * @brief server_socket module implementation
 *
 * Copyright © 2019 Octavo Systems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _SERVER_SOCKET_H_
#define _SERVER_SOCKET_H_

#include <arpa/inet.h>
#include "socket.hh"
#include "rtstatus.h"

class server_socket : public socket
{
  private:
    sockaddr_in_t addr;
    int client_handle;
 
  public:
    // constructor
    server_socket(unsigned short port, 
                  unsigned long addr = INADDR_ANY, 
                  unsigned short addr_family = AF_INET, 
                  int protocol_family = PF_INET,  
                  int type = SOCK_STREAM, 
                  int protocol = IPPROTO_TCP); 
    //destructor
    ~server_socket();
    rt_status_t open(); 

    // member functions
    void close();
    void close_client_handle();
    rt_status_t wait_for_connection();
    ssize_t send(void* send_data, unsigned int send_len);
    ssize_t recv(void* recv_data, unsigned int recv_len);
    rt_status_t send_wait(void* send_data, unsigned int send_len);
    rt_status_t recv_wait(void* recv_data, unsigned int recv_len);
    
};

#endif // _SERVER_SOCKET_H_
      
