/*
 * @file   socket.hh
 * @author David Weber <david.weber@octavosystems.com>
 * @date   02/14/2019
 *
 * @brief socket module implementation
 *
 * Copyright © 2019 Octavo Systems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _SOCKET_HH_
#define _SOCKET_HH_

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdbool.h>
#include "rtstatus.h"

typedef struct inet_addr_s
{
  unsigned long s_addr;
} inet_addr_t;

typedef struct sockaddr_in_s
{
  unsigned short sin_family;
  unsigned short sin_port;
  inet_addr_t sin_addr;
  char sin_zero[8];
} sockaddr_in_t;

class socket
{
  private:
    bool socket_opened;
    int protocol_family;
    int type;
    int protocol;

  protected:
    int socket_handle;

  public:
    socket(int domain = PF_INET, int type = SOCK_STREAM, 
           int protocol = IPPROTO_TCP);
    ~socket();

    rt_status_t open();
    void close();
    ssize_t send(int sockid, void* buf, unsigned int len, int flags = 0);
    ssize_t recv(int sockid, void* buf, unsigned int len, int flags = 0);
    rt_status_t send_wait(int sockid, void* buf, unsigned int len, int flags = 0);
    rt_status_t recv_wait(int sockid, void* buf, unsigned int len, int flags = 0);
};

#endif // _SOCKET_HH_
