/*
 * @file   ui_server.cc
 * @author David Weber <david.weber@octavosystems.com>
 * @date   02/10/2019
 *
 * @brief ui_server module implementation
 *
 * Copyright © 2019 Octavo Systems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include "socket.hh"
#include "server_socket.hh"
#include "msg_types.h"
#include "client_socket.hh"

char data[2048];

int main(int argc, char* argv[])
{
  msg_header_t msg_header;
  if (argc != 2)
  {
    fprintf(stderr, "Usage - %s <server port>\n", PROG_NAME);
    return -1;
  }
  int port = atoi(argv[1]);
  char* client_host = argv[2];
  int client_port = atoi(argv[3]);
  rt_status_t status;
  server_socket* ss = new server_socket(port);
  status = ss->open();
  if (status != SUCCESS)
  {
    fprintf(stderr, "unable to start server\n");
    return -1;
  }
  fprintf(stderr, "%s running...\n", PROG_NAME);
  while(1)
  {
    status = ss->wait_for_connection();
    if (status != SUCCESS)
    {
      perror("connection failed\n");
      return -1;
    }
    pid_t pid = fork();
    if (pid == -1)
    {
      perror("fork failed");
      return -1;
    }
    else if (pid > 0)
    {
      //parent closes client connection
      ss->close_client_handle();
    }
    else
    {
      pid = getpid();
      fprintf(stderr, "process %d forked to handle connection\n", pid);
      while(1)
      {
        // child process handles connection
        status = ss->recv_wait(&msg_header, sizeof(msg_header_t));
        if (status == SUCCESS)
        {
          size_t data_sz = msg_header.length - sizeof(msg_header_t);
          status = ss->recv_wait(&data, data_sz);
          if (status == SUCCESS)
          {
            fprintf(stderr, "pid %d: msg received -- sending to stdout\n", pid);
            fwrite(&msg_header, sizeof(msg_header_t), 1, stdout);
            fwrite(data, msg_header.length - sizeof(msg_header_t), 1, stdout);
          }
          else
          {
            fprintf(stderr, "pid %d: server recv failure - server process exiting\n", pid);
            fprintf(stderr, "pid %d: status = %d\n", pid, status);
            ss->close_client_handle();
            exit(0);
          }
        }
        else
        {
          fprintf(stderr, "pid %d: client disconnected - server process exiting\n", pid);
          fprintf(stderr, "pid %d: status = %d\n", pid, status);
          ss->close_client_handle();
          exit(0);
        }
      }
    }
  }
  delete(ss);
}

